<?php

declare(strict_types=1);

namespace LinkMobility\ShopwareSmsApiPlugin\Service;

use LinkMobility\ShopwareSmsApiPlugin\Exception\SmsApiRequestException;
use Psr\Http\Message\ResponseInterface;
use Shopware\Core\Checkout\Customer\CustomerEntity;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;

final class ContactService implements ContactServiceInterface
{
    public function __construct(
        private EntityRepository $customerRepository,
        private SmsApiServiceInterface $smsApiService
    ) {
    }

    public function create(CustomerEntity $customer, Context $context): void
    {
        $smsApiId = $this->createContact($customer, $context);

        if (null === $smsApiId) {
            if ('crud' === $context->getScope()) {
                return;
            }

            throw new SmsApiRequestException(
                'link-mobility-shopware-sms-api-plugin.exceptionMessage.phoneNumberOrEmailInvalid'
            );
        }

        $this->customerRepository->upsert([[
            'id' => $customer->getId(),
            'customFields' => ['link_mobility_shopware_sms_api_plugin_customer_id' => $smsApiId],
        ]], $context);
    }

    public function edit(
        CustomerEntity $customer,
        string $customerSmsApiId,
        Context $context
    ): void {
        if (!$this->contactExist($customerSmsApiId)) {
            return;
        }

        try {
            $this->smsApiService->request('contacts/' . $customerSmsApiId, 'POST', [
                'form_params' => $this->getContactParams($customer),
            ]);
        } catch (\Exception $e) {
            if (400 === $e->getCode()) {
                $this->delete($customer, $customerSmsApiId, $context);

                throw new SmsApiRequestException(
                    'link-mobility-shopware-sms-api-plugin.exceptionMessage.phoneNumberOrEmailInvalid'
                );
            }
        }

        if (!$this->contactExistByPhoneNumber($customer)) {
            $this->delete($customer, $customerSmsApiId, $context);

            throw new SmsApiRequestException(
                'link-mobility-shopware-sms-api-plugin.exceptionMessage.numberExist'
            );
        }
    }

    public function delete(
        CustomerEntity $customer,
        string $customerSmsApiId,
        Context $context
    ): void {
        if ('' === $customerSmsApiId) {
            return;
        }

        $this->smsApiService->request('contacts/' . $customerSmsApiId, 'DELETE');

        $this->clearCustomFields($customer, $context);
    }

    public function deleteContact(string $customerSmsApiId): void
    {
        $this->smsApiService->request('contacts/' . $customerSmsApiId, 'DELETE');
    }

    private function createContact(CustomerEntity $customer, Context $context): ?string
    {
        try {
            $response = $this->smsApiService->request('contacts', 'POST', [
                'form_params' => $this->getContactParams($customer),
            ]);
        } catch (\Exception $e) {
            if (400 === $e->getCode()) {
                return null;
            }
        }

        if (isset($response)) {
            $responseData = $this->jsonDecode($response);
        }

        if (!isset($responseData['id'])) {
            if ('crud' === $context->getScope()) {
                return null;
            }

            throw new SmsApiRequestException(
                'link-mobility-shopware-sms-api-plugin.exceptionMessage.connectError'
            );
        }

        return $responseData['id'];
    }

    private function contactExist(string $customerSmsApiId): bool
    {
        $response = $this->smsApiService->request('contacts', 'GET');

        $responseData = $this->jsonDecode($response);

        $contactsCollection = $responseData['collection'] ?? [];

        foreach ($contactsCollection as $contact) {
            if (isset($contact['id']) && $contact['id'] === $customerSmsApiId) {
                return true;
            }
        }

        return false;
    }

    private function contactExistByPhoneNumber(CustomerEntity $customer): bool
    {
        $phoneNumber = $customer->getDefaultBillingAddress()?->getPhoneNumber();

        $response = $this->smsApiService->request('contacts?q=' . $phoneNumber, 'GET');

        $responseData = $this->jsonDecode($response);
        $searchedNumber = $responseData['collection'][0]['id'] ?? null;

        $customFields = $customer->getCustomFields();
        $customerSmsApiId = $customFields['link_mobility_shopware_sms_api_plugin_customer_id'] ?? null;

        if (null === $searchedNumber && null === $customerSmsApiId) {
            return false;
        }

        if ($searchedNumber === $customerSmsApiId) {
            return true;
        }

        return false;
    }

    private function getContactParams(CustomerEntity $customer): array
    {
        $phoneNumber = $customer->getDefaultBillingAddress()?->getPhoneNumber();

        if (null === $phoneNumber) {
            return [];
        }

        return [
            'phone_number' => $phoneNumber,
            'email' => $customer->getEmail(),
            'first_name' => $customer->getFirstName(),
            'last_name' => $customer->getLastName(),
            'city' => $customer->getDefaultBillingAddress()?->getCity(),
            'birthday_date' => $customer->getBirthday()?->format('Y-m-d'),
        ];
    }

    private function jsonDecode(ResponseInterface $response): array
    {
        return json_decode($response->getBody()->getContents(), true);
    }

    private function clearCustomFields(CustomerEntity $customer, Context $context): void
    {
        $this->customerRepository->update([[
            'id' => $customer->getId(),
            'customFields' => [
                'link_mobility_shopware_sms_api_plugin_consent' => false,
                'link_mobility_shopware_sms_api_plugin_customer_id' => '',
            ], ]], $context);
    }
}
