<?php

declare(strict_types=1);

namespace LinkMobility\ShopwareSmsApiPlugin\Service\ScheduledTask;

use Doctrine\DBAL\Connection;
use LinkMobility\ShopwareSmsApiPlugin\Factory\SmsConfigFactoryInterface;
use LinkMobility\ShopwareSmsApiPlugin\Model\SmsConfig;
use LinkMobility\ShopwareSmsApiPlugin\Service\SmsSenderInterface;
use Shopware\Core\Checkout\Customer\CustomerEntity;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\Framework\MessageQueue\ScheduledTask\ScheduledTaskHandler;
use Symfony\Bridge\Monolog\Logger;

class AbandonedCartScheduledTaskHandler extends ScheduledTaskHandler
{
    public function __construct(
        EntityRepositoryInterface $scheduledTaskRepository,
        private Connection $connection,
        private SmsSenderInterface $smsSender,
        private EntityRepository $customerRepository,
        private SmsConfigFactoryInterface $smsConfigFactory,
        private Logger $logger,
        ) {
        parent::__construct($scheduledTaskRepository);
    }

    public static function getHandledMessages(): iterable
    {
        return [AbandonedCartScheduledTask::class];
    }

    public function run(): void
    {
        $smsConfig = $this->smsConfigFactory->createAbandonedCartConfig();
        if ($smsConfig->isActive()) {
            $this->handleAbandonedCarts($smsConfig);
        }
    }

    private function handleAbandonedCarts(SmsConfig $config): void
    {
        $fixedDate = new \DateTime('now');
        $fixedDate->modify('-' . $config->getDateModifier() . ' day');

        $fixedDateNextDay = new \DateTime($fixedDate->format('Y-m-d'));
        $fixedDateNextDay->modify('+ 1 day');

        $queryForUpdatedAtGivenTime = <<<'SQL'
                SELECT hex(`customer_id`) as `customer_id` FROM cart WHERE
                updated_at BETWEEN :fixedDate and :fixedDateNextDay AND hex(`customer_id`) IS NOT NULL ;
            SQL;

        try {
            $carts = $this->connection->fetchAllAssociative(
                $queryForUpdatedAtGivenTime,
                [
                    'fixedDate' => $fixedDate->format('Y-m-d'),
                    'fixedDateNextDay' => $fixedDateNextDay->format('Y-m-d'),
                ]
            );
        } catch (\Exception $e) {
            $this->logger->error($e->getMessage());

            return;
        }

        foreach ($carts as $cart) {
            /** @psalm-suppress InternalMethod */
            $context = Context::createDefaultContext();
            $criteria = new Criteria();
            $criteria->addAssociation('defaultBillingAddress');
            $criteria->addFilter(new EqualsFilter('id', $cart['customer_id']));

            /** @var CustomerEntity $customer */
            $customer = $this->customerRepository->search($criteria, $context)->first();

            $phoneNumber = $customer->getDefaultBillingAddress()?->getPhoneNumber();
            $customFields = $customer->getCustomFields();

            $consent = $customFields['link_mobility_shopware_sms_api_plugin_consent'] ?? false;

            if ($consent && null !== $phoneNumber) {
                $this->smsSender->sendCustomerAware($customer, $phoneNumber, $config->getMessage());
            }
        }
    }
}
