<?php

declare(strict_types=1);

namespace LinkMobility\ShopwareSmsApiPlugin\Service;

use LinkMobility\ShopwareSmsApiPlugin\Exception\SmsApiRequestException;
use LinkMobility\ShopwareSmsApiPlugin\Extractor\CustomerParameterExtractorInterface;
use LinkMobility\ShopwareSmsApiPlugin\Extractor\OrderParametersExtractorInterface;
use LinkMobility\ShopwareSmsApiPlugin\Extractor\ProductParameterExtractorInterface;
use LinkMobility\ShopwareSmsApiPlugin\Factory\SmsFactoryInterface;
use Shopware\Core\Checkout\Customer\CustomerEntity;
use Shopware\Core\Checkout\Order\OrderEntity;
use Shopware\Core\Content\Product\ProductEntity;

class SmsSender implements SmsSenderInterface
{
    public function __construct(
        private SmsApiServiceInterface $smsApiService,
        private OrderParametersExtractorInterface $orderParametersExtractor,
        private SmsFactoryInterface $smsFactory,
        private CustomerParameterExtractorInterface $customerParameterExtractor,
        private ProductParameterExtractorInterface $productParameterExtractor
    ) {
    }

    public function sendTestSms(int $phoneNumber): void
    {
        $sms = $this->smsFactory->createComplete((string) $phoneNumber, 'Test', 'hello');

        $response = $this->smsApiService->request('sms.do', 'POST', [
            'form_params' => $sms->toArray(),
        ]);
        $responseData = $response->getStatusCode();

        if (200 !== $responseData) {
            throw new SmsApiRequestException('link-mobility-shopware-sms-api-plugin.exceptionMessage');
        }
    }

    public function sendOrderStateChanged(
        OrderEntity $order,
        string $phoneNumber,
        string $rawSms
    ): void {
        $smsParameters = $this->orderParametersExtractor->extract($order);

        $sms = $this->prepareSms($smsParameters, $rawSms);

        $this->send($phoneNumber, $sms);
    }

    public function sendCustomerAware(
        CustomerEntity $customer,
        string $phoneNumber,
        string $rawSms
    ): void {
        $smsParameters = $this->customerParameterExtractor->extract($customer);

        $sms = $this->prepareSms($smsParameters, $rawSms);
        $this->send($phoneNumber, $sms);
    }

    public function sendProductAware(
        ProductEntity $product,
        string $phoneNumber,
        string $rawSms
    ): void {
        $smsParameters = $this->productParameterExtractor->extract($product);

        $sms = $this->prepareSms($smsParameters, $rawSms);

        $this->send($phoneNumber, $sms);
    }

    private function send(string $phoneNumber, string $sms): void
    {
        $sms = $this->smsFactory->createComplete($phoneNumber, 'Test', $sms);

        $response = $this->smsApiService->request('sms.do', 'POST', [
            'form_params' => $sms->toArray(),
        ]);

        $responseData = $response->getStatusCode();
        if (200 !== $responseData) {
            throw new SmsApiRequestException('link-mobility-shopware-sms-api-plugin.exceptionMessage');
        }
    }

    private function prepareSms(array $parameters, string $rawSms): string
    {
        foreach ($parameters as $name => $value) {
            if (!str_contains($rawSms, '{' . $name . '}')) {
                continue;
            }
            $rawSms = str_replace('{' . $name . '}', (string) $value, $rawSms);
        }

        return $rawSms;
    }
}
